
/* Copyright (C) Tom Horsley, 1998. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include "ntcli.h"

static char * progname = "basename";

int
main(int argc, char ** argv)
{
   NTCLIOptinfo oi;
   char * filearg;
   char * sufarg = NULL;
   char * extra = NULL;
   char * base;

   /* I'll be trying to wildcard expand things which might be on removable
   // devices, so tell the system to take a hike when it comes to displaying
   // error pop-up window's and just return an error code from the system
   // service instead.
   */
   SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);

   oi = NTCLIInitOptinfo(argc, argv, "");
   progname = NTCLIGetProgname(oi);
   for ( ; ; ) {
      int opt = NTCLIGetNextOption(oi);
      if (opt == -1) {
         char * msg = NTCLIErrorString(oi);
         if (msg != NULL) {
            fprintf(stderr, "%s\n", msg);
            exit(2);
         } else {
            break;
         }
      }
   }

   /* Now expand the arguments and process the files.
    */
   NTCLIExpandWildcards(oi);
   filearg = NTCLIGetNextPositionalArg(oi);
   if (filearg != NULL) {
      sufarg = NTCLIGetNextPositionalArg(oi);
      if (sufarg != NULL) {
         extra = NTCLIGetNextPositionalArg(oi);
         if (extra != NULL) {
            fprintf(stderr, "%s: Too many arguments\n", progname);
            exit(2);
         }
      }
   } else {
      fprintf(stderr, "%s: Missing argument(s)\n", progname);
      exit(2);
   }

   /* Figure out the base part of the name.
   */

   base = filearg + strlen(filearg) - 1;
   while ((base > filearg) && (*base == '\\')) {
      /* Remove trailing directory sep chars.
      */
      *base = '\0';
   }
   base = strrchr(filearg, '\\');
   if (base != NULL) {
      if (base != filearg) {
         ++base;
      }
   } else {
      base = filearg;
   }

   /* Now strip off the suffix if that was requested.
   */

   if (sufarg != NULL) {
      int baselen = strlen(base);
      int suflen = strlen(sufarg);
      if (suflen < baselen) {
         if (strcmp(&base[baselen-suflen], sufarg) == 0) {
            base[baselen-suflen] = '\0';
         }
      }
   }

   /* Now print the result.
   */
   puts(base);
   return 0;
}
