#include <ctype.h>
#include "ntcli.h"

/* Shell-style pattern matching for ?, \, [], and * characters.
   I'm putting this replacement in the public domain.

   Written by Rich $alz, mirror!rs, Wed Nov 26 19:03:17 EST 1986.

   Case sensitive mods made by Tom Horsley for use with Windows NT */

/* The character that inverts a character class; '!' or '^'.  */
/* NOTE that ^ is more like most U**xen, but ^ is the escape char
// in the Windows NT CMD prompt, so it is kind of inconvenient
// to use, and ! is selected as the next best thing.
*/
#define INVERT '!'

static int star(char *string, char *pattern, int case_sensitive);

/* Return nonzero if `string' matches Unix-style wildcard pattern
   `pattern'; zero if not.  */

DllExport int
NTCLIWildMatch(char *string, char *pattern, int case_sensitive)
{
  int prev;             /* Previous character in character class.  */
  int matched;          /* If 1, character class has been matched.  */
  int reverse;          /* If 1, character class is inverted.  */

  for (; *pattern; string++, pattern++)
    switch (*pattern)
      {
      case '\\':
        /* Literal match with following character; fall through.  */
        pattern++;
      default:
        if (*string != *pattern)
          if (! case_sensitive) {
             char c1 = *string;
             char c2 = *pattern;
             c1 = tolower(c1);
             c2 = tolower(c2);
             if (c1 != c2) return 0;
          } else {
            return 0;
          }
        continue;
      case '?':
        /* Match anything.  */
        if (*string == '\0')
          return 0;
        continue;
      case '*':
        /* Trailing star matches everything.  */
        return *++pattern ? star (string, pattern, case_sensitive) : 1;
      case '[':
        /* Check for inverse character class.  */
        reverse = pattern[1] == INVERT;
        if (reverse)
          pattern++;
        if (case_sensitive) {
          for (prev = 256, matched = 0; *++pattern && *pattern != ']';
               prev = *pattern)
            if (*pattern == '-'
                ? *string <= *++pattern && *string >= prev
                : *string == *pattern)
              matched = 1;
        } else {
          char c1, c2;
          c2 = *string;
          c2 = tolower(c2);
          for (prev = 256, matched = 0; *++pattern && *pattern != ']';
               prev = tolower(*pattern))
            if ((c1 = tolower(*pattern)) == '-') {
              ++pattern;
              c1 = *pattern;
              c1 = tolower(c1);
              matched |= (c2 <= c1 && c2 >= prev);
            } else {
              matched |= (c2 == c1);
            }
        }
        if (matched == reverse)
          return 0;
        continue;
      }

  return *string == '\0';
}

static int
star (char *string, char *pattern, int case_sensitive)
{
  while (NTCLIWildMatch(string, pattern, case_sensitive) == 0)
    if (*++string == '\0')
      return 0;
  return 1;
}
